#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <stdbool.h>

//#define Dcols 64
//#define Drows 48
//#define DpixelCount Dcols * Drows
//#define DpixelsPerByte 8
//#define DframeSize DpixelCount / DpixelsPerByte


#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitToggle(value, bit) ((value) ^= (1UL << (bit)))
#define bitWrite(value, bit, bitvalue) ((bitvalue) ? bitSet(value, bit) : bitClear(value, bit))

void decode(uint16_t *inBuffer, uint8_t *outBuffer, uint8_t columns, uint8_t rows, uint8_t pixelsPerByte) {
    uint16_t pixelCount;
    //uint16_t frameSize;
    uint16_t inData;
    uint16_t prevCounter = 0;
    uint16_t inPtr;
    pixelCount = columns * rows;
    //frameSize = pixelCount / pixelsPerByte;
    inPtr = 0;
    
    inData = inBuffer[inPtr];
    inPtr ++;
    
    while (prevCounter < pixelCount) {
        bool curPixel;
        uint16_t counter;
        curPixel = (bool)(0x8000 & inData);
        counter = (0x7FFF & inData) + 1;
        for (uint16_t pixel = prevCounter; pixel < (prevCounter + counter); pixel++) {
            uint8_t bitIndex;
            uint16_t arrayIndex;
            arrayIndex = pixel / pixelsPerByte;
            bitIndex = pixelsPerByte - 1 - (pixel % pixelsPerByte);
            bitWrite(outBuffer[arrayIndex], bitIndex, curPixel);
            //printf("pixel: %d\tarrIdx: %d\tbitIdx: %d\tcolor: %d\tprevCnt: %d\tcnt: %d\n", pixel, arrayIndex, bitIndex, curPixel,prevCounter, counter);
        }
        prevCounter += counter;
        //printf("color: %d\tcounter: %d\n",curPixel, counter);
        //fread(&inData, sizeof(inData), 1, ptrInFile);
        inData = inBuffer[inPtr];
        inPtr++;
    }
}

int main(int argc, char *argv[]) {
    FILE *ptrInFile;
    FILE *ptrOutFile;
    
    uint16_t inBuffer[2000];
    uint8_t outBuffer[500];
    uint16_t inData;
    
    uint8_t colums = 64;
    uint8_t rows = 48;
    uint8_t pixelsPerByte = 8;
    
    if (argc != 3) {
        printf("Usage: enc frame.bin result.bin\n");
        printf("./enc frame000.bin result000.bin\n");
        exit(-1);
    }
    
    ptrInFile = fopen(argv[1], "rb");
    
    uint16_t i = 0;
    while (!feof(ptrInFile)) {
        fread(&inData, sizeof(inData), 1, ptrInFile);
        inBuffer[i] = inData;
        i++;
    }
    fclose(ptrInFile);
    
    //printf("Start decode");
    decode(inBuffer, outBuffer, colums, rows, pixelsPerByte);
    //printf("Completed decode");
    
    ptrOutFile = fopen(argv[2], "wb");
    fwrite(&outBuffer, colums* rows / pixelsPerByte, 1, ptrOutFile);
    fclose(ptrOutFile);
    return 0;
}

/*
int main(int argc, char *argv[]) {
    FILE *ptrInFile;
    FILE *ptrOutFile;
    
    uint8_t inFrame[DframeSize];
    uint16_t inData;
    uint16_t prevCounter = 0;
    
    if (argc != 3) {
        printf("Usage: enc frame.bin result.bin\n");
        printf("./enc frame000.bin result000.bin\n");
        exit(-1);
    }
    
    ptrInFile = fopen(argv[1], "rb");
    ptrOutFile = fopen(argv[2], "wb");
    
    fread(&inData, sizeof(inData), 1, ptrInFile);

    while (!feof(ptrInFile)) {
        bool curPixel;
        uint16_t counter;
        curPixel = (bool)(0x8000 & inData);
        counter = (0x7FFF & inData) + 1;
        for (uint16_t pixel = prevCounter; pixel < (prevCounter + counter); pixel++) {
            uint8_t bitIndex;
            uint16_t arrayIndex;
            arrayIndex = pixel / DpixelsPerByte;
            bitIndex = DpixelsPerByte - 1 - (pixel % DpixelsPerByte);
            bitWrite(inFrame[arrayIndex], bitIndex, curPixel);
            //printf("pixel: %d\tarrIdx: %d\tbitIdx: %d\tcolor: %d\tprevCnt: %d\tcnt: %d\n", pixel, arrayIndex, bitIndex, curPixel,prevCounter, counter);
        }
        prevCounter += counter;
        //printf("color: %d\tcounter: %d\n",curPixel, counter);
        fread(&inData, sizeof(inData), 1, ptrInFile);
    }
    fwrite(&inFrame, sizeof(inFrame), 1, ptrOutFile);
    fclose(ptrInFile);
    fclose(ptrOutFile);
    return 0;
}
*/
